import streamlit as st
from javascript import require, On
import datetime
import asyncio
import math
import json
import csv
    
def makeBot(x_coord, y_coord, z_coord, data):
    mineflayer = require('mineflayer')
    pathfinder = require('mineflayer-pathfinder')
    goals = require('mineflayer-pathfinder').goals
    mineflayerViewer = require('prismarine-viewer').mineflayer
    Vec3 = require("vec3").Vec3
    windows = require('prismarine-windows')(data["version"])
    Item = require('prismarine-item')(data["version"])
    armorManager = require("mineflayer-armor-manager")
    tpsPlugin = require('mineflayer-tps')(mineflayer)
    
    bot = mineflayer.createBot({
        'host': data["server_ip"],
        'port': data["server_port"],
        'username': data["bot_name"],
        'password': data["password"],
        'auth': data["auth"],
        'version': data["version"],
        'checkTimeoutInterval': data["check_timeout_interval"],
    })
    
    def add_values_to_csv(data, bot):
        now = datetime.datetime.now()
        server_info = f"{data['server_ip']}:{data['server_port']}"
        start_x = str(bot.entity.position.x)
        start_y = str(bot.entity.position.y)
        start_z = str(bot.entity.position.z)
        
        delivered_item = data["items_name"]

        distance = math.sqrt((x_coord - start_x)**2 + (z_coord - start_z)**2)

        row = [now, server_info, start_x, start_y, start_z, x_coord, y_coord, z_coord, distance, delivered_item]

        with open('logs\\analytics.csv', 'a', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(row)
    
    def logPlayers(bot):
        with open("logs\players.log", "w") as x:
            x.write(f'{str(bot.players)}\n')
        
    bot.loadPlugin(pathfinder.pathfinder)
    bot.loadPlugin(armorManager)
    
    @On(bot, 'login')
    def handle_login(*args):
        mineflayerViewer(bot, { "firstPerson": True, "port": data["viewer_port"] })
        global mcData
        mcData = require('minecraft-data')(bot.version)
        movements = pathfinder.Movements(bot, mcData)
        
        try:
            bot.armorManager.equipAll()
        except Exception as e:
            print(e)
        
        logPlayers(bot)

        movements.canDig = False
        
        bot.pathfinder.setMovements(movements)
        
        @On(bot, "death")
        def death(this):
            bot.end()
            bot.viewer.close()
            st.toast("Bot died... stopping bot!")
        
        @On(bot, "kicked")
        def kicked(this, reason, *a):
            bot.end()
            bot.viewer.close()
            st.toast("Kicked from server... stopping bot!")
                
        def itemByName(items, name):
            item = None
            for i in range(len(items)):
                item = items[i]
                if item and item['name'] == name:
                    return item
                return None
        
        def GetItems():
            x = data["chest_coords"][0]
            y = data["chest_coords"][1]
            z = data["chest_coords"][2]
            locaton = bot.pathfinder.setGoal(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
            
            chestToOpen = bot.findBlock({
                'matching': [mcData.blocksByName[name].id for name in [f'{str(data["chest_type"]).lower()}']],
                'maxDistance': 10,
            })
            
            if chestToOpen:

                chest = bot.openContainer(bot.blockAt(Vec3(-62, 72, 47)))
                
                item = itemByName(chest.containerItems(), data["items_name"])
                
                if item:
                    try:
                        windows.withdraw(item.type, None, data['items_count'])
                    except Exception as err:
                        bot.chat(f"unable to withdraw {data['items_count']} {item.name}")
                else:
                    bot.chat(f"unknown item {data['items_name']}")
                
                asyncio.sleep(5)
                
                chest.close()                
                
            else:
                bot.chat("Can't find the chest")
        
        def GoToLocation():
                x = float(x_coord)
                y = float(x_coord)
                z = float(z_coord)
                locaton =  bot.pathfinder.setGoal(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
        
        def DepositItems():

            foundChest = False
            
            while not foundChest:
                
                chestToOpen = bot.findBlock({
                    'matching': [mcData.blocksByName[name].id for name in ['chest']], 
                    'maxDistance': data["chest_range"],
                })
            
                if not chestToOpen and foundChest == False:
                    st.toast("No delivery chest found")
                    break
                    
                if chestToOpen.position.x:
                    x = chestToOpen.position.x
                    y = chestToOpen.position.y  
                    z = chestToOpen.position.z

                    locaton = bot.pathfinder.setGoal(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
                    
                    try:
                        chest = bot.openContainer(chestToOpen)
                    except Exception:
                        print(Exception)
                        continue
                    
                    item = next((item for item in chest.slots if item and item.name == f'{data["items_name"]}'), None)
                    windows.deposit(item, "null", "null", "null")
                    
                    chest.close()

                    foundChest = True
                    
                    break
                
            add_values_to_csv(data, bot)
            
        GetItems()
            



import streamlit as st
from st_on_hover_tabs import on_hover_tabs
import streamlit_toggle as tog
from streamlit_extras.add_vertical_space import add_vertical_space
from streamlit_extras.stateful_button import button
from streamlit_extras.no_default_selectbox import selectbox
from streamlit_extras.grid import grid
from javascript import require, On
import pandas as pd
import numpy as np
import math
import datetime
import requests
import json
import asyncio
from bot import makeBot

st.set_page_config(
    page_title="Bot controller",
    page_icon="🕹",
    layout="centered",
    initial_sidebar_state="auto",
)

with open('data\data.json', 'r') as f:
    data = json.load(f)
        
def webmode():
    st.markdown("<style>" + open("frontend\styles.css").read() + "</style>", unsafe_allow_html=True)

    with st.sidebar:
        tabs = on_hover_tabs(
            tabName=["Settings", "Dashboard", "Analytics"],
            iconName=["settings", "dashboard", "analytics"],
            styles={
                "navtab": {
                    "background-color": "#111",
                    "color": "#818181",
                    "font-size": "18px",
                    "transition": ".3s",
                    "white-space": "nowrap",
                    "text-transform": "uppercase",
                },
                "tabOptionsStyle": {
                    ":hover :hover": {"color": "white", "cursor": "pointer"}
                },
                "iconStyle": {
                    "position": "fixed",
                    "left": "7.5px",
                    "text-align": "left",
                },
                "tabStyle": {
                    "list-style-type": "none",
                    "margin-bottom": "30px",
                    "padding-left": "30px",
                },
            },
            key="1",
        )

    if tabs == "Dashboard":
        st.title("Bot controller")
            
        col1, col2, col3, col4, col5 = st.columns([2, 2, 2, 2, 2])

        with col1:
            x_coord = st.text_input("x", label_visibility="collapsed", placeholder="X coord")
                
        with col2:
            y_coord = st.text_input("y", label_visibility="collapsed", placeholder="Y coord")
            
        with col3:
            z_coord = st.text_input("z", label_visibility="collapsed", placeholder="Z coord")

        with col5:
            # def isViewerRunning():
            #     try:
            #         url = "http://127.0.0.1:1000"
            #         get_url = requests.get(url)
            #         code = get_url.status_code
                    
            #         if code == 200:
            #             return True
            #     except requests.exceptions.RequestException:
            #         return False
                
            def openViewer():
                render = st.markdown('<iframe width="700rem" height="400rem" src="http://127.0.0.1:1000" frameborder="0" scrolling="auto" allowfullscreen id="render"></iframe>', unsafe_allow_html=True)

            # if isViewerRunning:
            #     start_dropoff_bot = st.button("Deliver item", disabled=True)
            # else:
            start_dropoff_bot = st.button("Deliver item")
            
        with col4:
            # if isViewerRunning:
            #     prismarineViewer_button = st.button("Open bot view", key="viewer_button", on_click=openViewer)
            # else:
            prismarineViewer_button = st.button("Open bot view", key="viewer_button", on_click=openViewer)
            if start_dropoff_bot:
                try:
                    makeBot(x_coord, y_coord, z_coord, data)
                    
                    st.toast("Bot started!", icon="✅")

                except ValueError:
                    st.toast("Please enter a valid float number", icon="🚨")
            
        
    if tabs == "Settings":
        st.title("Bot settings")
        bot_name = st.text_input(label="Bot name", placeholder="BOT", value=data["bot_name"])

        col1, col2 = st.columns((2, 5))
        
        with col1:
            server_ip = st.text_input(label="Server ip", placeholder="127.0.0.1", value=data["server_ip"])
            viewer_ip = st.text_input(label="Viewer ip", placeholder="127.0.0.1", value=data["viewer_ip"])
            password = st.text_input(label="Password", type="password", value=data["password"])
            chest_type = selectbox("Select the chest type", ["chest", "trapped_chest"], no_selection_label=f"Selected: {data['chest_type']}")
                        
            def save_changes():
                with open('data\data.json', 'r') as file:
                    data = json.load(file)
                
                data['server_ip'] = server_ip
                data['server_port'] = server_port
                data['bot_name'] = bot_name
                data['quit_on_low_health'] = healt_toggle
                data['low_health_threshold'] = range_slider
                data['password'] = password
                data['auth'] = auth
                data['viewer_ip'] = viewer_ip
                data['viewer_port'] = viewer_port
                data['chest_type'] = chest_type
                data['chest_range'] = chest_range
                
                with open('data\data.json', 'w') as file:
                    json.dump(data, file, indent=4)


        with col2:
            server_port = st.text_input(label="Server port", placeholder="25565", value=data["server_port"])
            viewer_port = st.text_input(label="Viewer port", placeholder="2000", value=data["viewer_port"])
            
            auth = selectbox("Select an authentication method", ["Microsoft", "Cracked"], no_selection_label=f"Selected: {data['auth']}")
            chest_range = st.text_input(label="Chest range", placeholder="100", value=data["chest_range"])

        st.markdown("""---""")
        st.header("WIP section")
        add_vertical_space(1)
        
        wip_col1, wip_col2 = st.columns((2, 5))
        
        with wip_col1:
            healt_toggle = tog.st_toggle_switch(label="Quit on low health", 
                        key="Key1",
                        default_value=data["quit_on_low_health"], 
                        label_after = False, 
                        inactive_color = '#D3D3D3', 
                        active_color="#11567f", 
                        track_color="#29B5E8",
                        )
            
        with wip_col2:
            range_slider = st.slider(label=" ", label_visibility="collapsed" , min_value=1, max_value=19, value=data["low_health_threshold"], disabled=not healt_toggle)

        add_vertical_space(2)
        
        save_button = st.button(label="Save changes")
        
        if save_button:
            save_changes()
            st.toast("Saved changes!")
    
    if tabs == 'Analytics':
        st.title("Delivery analytics")
        st.header("WIP - Placeholder values only")
        csv_df = pd.DataFrame(np.random.randn(20, 3), columns=["a", "b", "c"])
        my_grid = grid(2, [2, 4, 1], 1, 4, vertical_align="bottom")
        
        my_grid.dataframe(csv_df, use_container_width=True)
        my_grid.line_chart(csv_df, use_container_width=True)

webmode()



import os
import sys
from .console_ui import consoleui
import click

@click.command()
@click.option("--console", default="", help="Use console as main type of input")
@click.option("--gui", default="", help="Use Web Gui as main type of input")
def run(console, gui):
    is_linux = sys.platform.startswith('linux')
    is_mac = sys.platform == 'darwin'
    is_windows = os.environ.get('windir')
    
    
    if console != "":
        consoleui()
    if gui != "":
        os.system("streamlit run web_ui.py")
    
    if console == "" and gui == "":
        if is_linux or is_mac:
            if os.environ.get('DISPLAY','') == '':
                print('No GUI on Linux/Mac, running console UI')  
                consoleui()
            else:
                print('Linux/Mac GUI found, running Streamlit')
                os.system("streamlit run web_ui.py")

        elif is_windows:
            print('Windows found, running Streamlit')
            os.system("streamlit run web_ui.py") 

        else:
            print('Unknown platform, defaulting to console UI')
            consoleui()
            
run()


#!/usr/bin/python
# -*- coding: utf-8 -*-
def consoleui():
  # Import necessary modules
  from javascript import require, On  
  import time
  import contextlib
  import io
  import sys
  import click
  import json
  import os
  from tqdm import tqdm
  from datetime import datetime
  import subprocess
  import requests
  import pkg_resources
  import structlog
  from threading import Thread

  def clear():
    if os.name == 'nt':
      _ = os.system('cls')
    else:
      _ = os.system('clear')
  clear()



  logger = structlog.get_logger()



  def nodeCheck():
    global node_version
    result = subprocess.run(["node", "--version"], 
                              stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE,
                              text=True)
    node_version = result.stdout.strip()
    # Remove leading 'v'
    node_version = node_version[1:] if node_version.startswith('v') else node_version
    # Remove periods
    node_version = node_version.replace('.', '')
    if int(node_version[:2]) >= 18:
      logger.info(f"Detected node version {node_version[:2]} witch is supported!")
    else:
      logger.warning(f"Detected node version {node_version[:2]} witch is NOT supported!\nThis may cause problems. Please update to node 18 or above!")
      time.sleep(7)
    
  nodeCheck()



  def updateCheck():
      package = 'opendeliverybot'  # replace with the package you want to check
      response = requests.get(f'https://pypi.org/pypi/{package}/json')
      global latest_version
      global current_version
      latest_version = response.json()['info']['version']
      current_version = pkg_resources.get_distribution(package).version
      if current_version != latest_version:
        logger.warning("You are not using the latest version!\nConsider updating with:\npip install -U opendeliverybot")
        time.sleep(7)
    
  updateCheck()



  @contextlib.contextmanager
  def nostdout():
      save_stdout = sys.stdout
      sys.stdout = io.StringIO()
      yield
      sys.stdout = save_stdout



  # Initialize mineflayer bot and plugins
  pbar = tqdm(total=7)
  pbar.set_description("installing javascript libaries", refresh=True)
  with nostdout():
    mcData = require('minecraft-data')
  pbar.update(1)
  with nostdout():
    mineflayer = require('mineflayer')
  pbar.update(1)
  with nostdout():
    pathfinder = require('mineflayer-pathfinder')
  pbar.update(1)
  with nostdout():
    goals = require('mineflayer-pathfinder').goals
  pbar.update(1)
  with nostdout():
    mineflayerViewer = require('prismarine-viewer').mineflayer
  pbar.update(1)
  with nostdout():
    Vec3 = require("vec3").Vec3
  pbar.update(1)
  with nostdout():
    armorManager = require("mineflayer-armor-manager")
  pbar.update(1)
  time.sleep(2)
  pbar.close()
  clear()



  # Function for implementing the loading animation
  def load_animation(text):
    
      # String to be displayed when the application is loading
      load_str = text
      ls_len = len(load_str)
    
    
      # String for creating the rotating line
      animation = "|/-\\"
      anicount = 0
        
      # used to keep the track of
      # the duration of animation
      counttime = 0        
        
      # pointer for travelling the loading string
      i = 0                     
    
      while (counttime != 100):
            
          # used to change the animation speed
          # smaller the value, faster will be the animation
          time.sleep(0.075) 
                                
          # converting the string to list
          # as string is immutable
          load_str_list = list(load_str) 
            
          # x->obtaining the ASCII code
          x = ord(load_str_list[i])
            
          # y->for storing altered ASCII code
          y = 0                             
    
          # if the character is "." or " ", keep it unaltered
          # switch uppercase to lowercase and vice-versa 
          if x != 32 and x != 46:             
              if x>90:
                  y = x-32
              else:
                  y = x + 32
              load_str_list[i]= chr(y)
            
          # for storing the resultant string
          res =''             
          for j in range(ls_len):
              res = res + load_str_list[j]
          
          
          # displaying the resultant string
          text = str("\r"+res + animation[anicount]).center(20)
          sys.stdout.write(text)
          sys.stdout.flush()
    
          # Assigning loading string
          # to the resultant string
          load_str = res
    
            
          anicount = (anicount + 1)% 4
          i =(i + 1)% ls_len
          counttime = counttime + 1
        
      # for windows OS
      clear()

  load_animation(f"starting open delivery bot... ")

  @click.command()
  @click.option("--username", help="Username for login.")
  @click.option("--password", help="Password for login.")  
  @click.option("--host", prompt=True, help="Hostname or IP address of server.")
  @click.option("--port", default=25565, help="Port number of server.")
  @click.option("--auth", default="microsoft", help="Authentication method.")
  @click.option("--version", default="auto", help="Game version.",prompt=True)
  @click.option("--check_timeout", default=600000, help="Timeout interval for checks.")
  @click.option("--viewer_port", default=8000, help="Port for viewer.")
  @click.option("--goto", default=["100", "100", "100"], multiple=True, help="Coordinates to go to.", prompt=True)
  @click.option("--chest_range", default=100, help="Range to search for chests.")
  @click.option("--init_chest_type", default="chest", help="Type of chest to look for when starting.")
  @click.option("--init_chest_cords", default=["100", "100", "100"], multiple=True, help="Coordinates to base chest.", prompt=True)  
  @click.option("--init_items_name", default="SchulkerBox", help="Name of items to get from the base chest.", prompt=True)
  @click.option("--init_items_count", default=1, help="Number of items to get from the base chest.",prompt=True)
  @click.option("--recipient_username", default="OpenDeliveryBot", help="Username to deliver to.", prompt=True)
  @click.option("--quit_on_low_health", default=True, help="Disconect the bot if the bot is on low health")
  @click.option("--low_health_threashold", default=10, help="How low the health must be for the bot to quit")
  @click.option("--armor_equip", default=True, help="If the bot needs to equip available armor.")
  def main(username, password, host, port, auth, version, check_timeout, viewer_port, goto, chest_range, init_chest_type, init_chest_cords, init_items_name, init_items_count, recipient_username, quit_on_low_health, low_health_threashold, armor_equip):

    DEV = False
    # Create version
    if version == "auto":
      version = False

    # Create bot instance 
    bot = mineflayer.createBot({
      'host': host,
      'port': port,
      'username': username,
      'password': password,
      'auth': auth,
      'hideErrors': True,
      'version': version,
      'checkTimeoutInterval': check_timeout,
    })

    # Load plugins
    bot.loadPlugin(pathfinder.pathfinder)
    bot.loadPlugin(armorManager)
    clear()
    # Login handler 
    @On(bot, 'login')
    def handle_login(*args):
      mineflayerViewer(bot, { "port": viewer_port })
      clear()
      logger.info(f'\n\nStarted Open Delivery Bot\nNode version: {node_version[:2]}\nCurrent version: {current_version}\nLatest version: {latest_version}\n\n')
      time.sleep(1)
      logger.info(f'Logged in as: {bot.username}')
      logger.info(f'Joined {host}:{port}')
      logger.info(f'Cordinates: {str(float(bot.entity.position.x))}, {str(float(bot.entity.position.y))}, {str(float(bot.entity.position.z))}')
      logger.info(f'Live webview is running at: http://localhost:{viewer_port}/')
      global mcData
      mcData = require('minecraft-data')(bot.version)
      movements = pathfinder.Movements(bot, mcData)
      
      
      # if quit_on_low_health == True:
      #     @On(bot, "health")
      #     def health(this):
      #         global botHealth
      #         botHealth = 20
      #         if bot.health >= botHealth:
      #             botHealth = bot.health
      #             return
              
      #         botHealth = bot.health
      #         if botHealth < low_health_threashold:
      #             bot.quit()

      # if armor_equip == True:
      #   try:
      #     bot.armorManager.equipAll()
      #   except Exception as e:
      #     pass
      
      
      with open("playes.log", "w") as x:
        x.write(str(bot.players))
      
      with open("cords.log", "a") as x:
        x.write(f"{datetime.now()}  |  {host}:{port}  |  X:{str(bot.entity.position.x)}, Y:{str(bot.entity.position.y)}, Z:{str(bot.entity.position.z)}")
      
      # Disable digging if needed
      movements.canDig = False

      bot.pathfinder.setMovements(movements)
      
      # Main login logic
      if DEV == True:
        GetItems(init_items_name, init_items_count)
        GoToLocation(goto)
        DepositItems("")
        Respawn()
        
        print(", ".join([e.name for e in bot.taskManager.GetWholeQueue()]))
      





    # Function to find and open the init chest
    def GetItems(items, count):
      
      if init_chest_cords == ["0", "0", "0"]:
        

        foundChest = False
        
        while not foundChest:
            
          # Find nearby chest
          
          chestToOpen = bot.findBlock({
              'matching': [mcData.blocksByName[name].id for name in [f'{str(init_chest_type).lower()}']],
              'maxDistance': 100,
          })
          
          if not chestToOpen and foundChest == False:
            bot.chat('no chest found')
            continue
            
          # Go to chest location  
          if chestToOpen.position.x:
            x = chestToOpen.position.x
            y = chestToOpen.position.y  
            z = chestToOpen.position.z

            bot.chat(f'/tell {recipient_username} [OPEN DELIVERY BOT] Found a nearby chest to deliver: [{str(x)}, {str(y)}, {str(z)}]')
            locaton = bot.pathfinder.goto(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
            
            try:
              chest = bot.openContainer(chestToOpen)  
            except Exception:    
              bot.chat(f'/tell {recipient_username} [OPEN DELIVERY BOT] Please place a new chest in my location.')
              continue
                
            print(chest.containerItems())
            item = next((item for item in chest.slots if item and item.name == f'{init_items_name}'), None)
            time.sleep(10)
            chest.close()
            bot.chat(f'/tell {recipient_username} [OPEN DELIVERY BOT] Items have been successfully delivered at: [{str(x)}, {str(y)}, {str(z)}] on [{datetime.now()}]')
            foundChest = True
            break
      else:
        x = init_chest_cords[0]
        y = init_chest_cords[1]
        z = init_chest_cords[2]
        locaton = bot.pathfinder.goto(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)






    # Function to go to cordinates
    def GoToLocation(cordinates):
        x = cordinates[0]
        y = cordinates[1]
        z = cordinates[2]
        bot.chat(f'/tell {recipient_username} [OPEN DELIVERY BOT] Going to: [{str(x)}, {str(y)}, {str(z)}]')
        locaton = bot.pathfinder.goto(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
        












    # Function to find and open chest  
    def DepositItems(items):

      foundChest = False
      
      while not foundChest:
          
        # Find nearby chest
        chestToOpen = bot.findBlock({
            'matching': [mcData.blocksByName[name].id for name in ['chest']], 
            'maxDistance': chest_range,
        })
        
        if not chestToOpen and foundChest == False:
          bot.chat('no chest found')
          continue
          
        # Go to chest location  
        if chestToOpen.position.x:
          x = chestToOpen.position.x
          y = chestToOpen.position.y  
          z = chestToOpen.position.z

          bot.chat(f'/tell {recipient_username} [OPEN DELIVERY BOT] Found a nearby chest to deliver: [{str(x)}, {str(y)}, {str(z)}]')
          locaton = bot.pathfinder.goto(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
          
          try:
            chest = bot.openContainer(chestToOpen)  
          except Exception:    
            bot.chat(f'/tell {recipient_username} [OPEN DELIVERY BOT] Please place a new chest in my location.')
            continue
              
          print(chest.containerItems())
          time.sleep(10)
          chest.close()
          bot.chat(f'/tell {recipient_username} [OPEN DELIVERY BOT] Items have been successfully delivered at: [{str(x)}, {str(y)}, {str(z)}] on [{datetime.now()}]')
          foundChest = True
          break
          

    # Function to find lava and selfdiscructs
    def Respawn():

      foundLava = False
      
      while not foundLava:
          
        # Find nearby lava
        LavaToFind = bot.findBlock({
            'matching': [mcData.blocksByName[name].id for name in ['lava']], 
            'maxDistance': 200,
        })
        
        if not LavaToFind and foundLava == False:
          bot.chat('no chest found')
          continue
          
        # Go to lava location  
        if LavaToFind.position.x:
          x = LavaToFind.position.x
          y = LavaToFind.position.y  
          z = LavaToFind.position.z
          locaton = bot.pathfinder.goto(pathfinder.goals.GoalNear(x, y, z, 1), timeout=60)
          break



    # Handler for bot end  
    @On(bot, 'end')
    def handle_end(*args):
      logger.info('Bot ended!', args)


  #run main
  main()